# Common parameters

set val(a_sn)		[expr $val(sn)-$val(dup)]

set val(chan)       	Channel/WirelessChannel
set val(prop)       	Propagation/TwoRayGround
set val(netif)      	Phy/WirelessPhy
set val(mac)        	Mac/802_11
set val(ifq)        	Queue/DropTail/PriQueue
set val(ll)         	LL
set val(ant)        	Antenna/OmniAntenna
set val(ifqlen)         50		;# max packet in ifq

set val(adhocRouting)   BCAST
set val(SLPport) 	18
set val(MulticastAddr) 	0xE000000
set val(extra)		20.00		;# wait for that amount after simulation ends
set val(motionFile)    	"slp_tools/slpMotion/$val(runDirectory)/$val(scenario).motion" 
set val(outputFile)    	"slp_tools/slpTraces/$val(runDirectory)/cache/$val(scenario).tr" 
############################################################################## 
# minimize memory requirements by removing all irrelevant packet headers
remove-packet-header PGM PGM_SPM PGM_NAK Pushback NV LDP MPLS rtProtoLS Ping
remove-packet-header TFRC TFRC_ACK Diffusion RAP TORA IMEP MIP
remove-packet-header IPinIP Encap HttpInval MFTP SRMEXT SRM aSRM
remove-packet-header mcastCtrl CtrMcast rtProtoDV GAF Snoop TCPA TCP IVS
remove-packet-header Resv UMP Src_rt

# selectively allow only one of the following: AODV, SR, BCAST
remove-packet-header AODV
remove-packet-header SR
# remove-packet-header BCAST
############################################################################## 


# Initialize Global Variables and Procedures

#Set up Random Number Generators

#seed the defaultRNG
$defaultRNG seed 9999

set arrivalRNG [new RNG]
set nnodeRNG [new RNG]
set serviceRNG [new RNG]

for {set j 1} {$j < $val(seed)} {incr j} {
 $arrivalRNG next-substream
 $nnodeRNG next-substream
 $serviceRNG next-substream
}
set arrival_ [new RandomVariable/Exponential]
$arrival_ set avg_ $val(exp_avg)
$arrival_ use-rng $arrivalRNG

set nnode_ [new RandomVariable/Uniform]
$nnode_ set min_ 0
$nnode_ set max_ [expr $val(nn)-1]
$nnode_ use-rng $nnodeRNG

set service_ [new RandomVariable/Uniform]
$service_ set min_ 0
if {$val(dup) == 0} {
  $service_ set max_ [expr $val(a_sn)-1]
} else { 
  $service_ set max_ $val(a_sn)
}
$service_ use-rng $serviceRNG

# Initialize array of nodes to zero (no nodes used yet)
for {set i 0} {$i < $val(nn) } {incr i} {
  set array($i) 0
}

# Procedure gets a random node number from the list of nodes
# and set its array position to 1, so it won't be polled again
proc get_snode {nnode_ name} {
  upvar $name array
  while (1) {
    set i [expr round([$nnode_ value])]
    if {$array($i) == 0} {
      set array($i) 1
      return $i 
    }
  }
}

# Procedure gets a random node number from the list of nodes
# and set its array position to 2, so it won't be polled again
proc get_cnode {nnode_ name} {
  upvar $name array
  while (1) {
    set i [expr round([$nnode_ value])]
    if {$array($i) == 0} {
      set array($i) 2
      return $i 
    }
  }
}

# Procedure gets an exponential arrival time (for request)
proc get_time {arrival_ stop} {
  set time [$arrival_ value];
  while {$time >= $stop || $time<1000} {
    set time [$arrival_ value];     
  }
  return $time;
}

# create simulator instance

set ns_		[new Simulator]

# setup topography object

set topo	[new Topography]

# create trace object for ns

set tracefd	[open $val(outputFile) w]

# $ns_ use-newtrace
$ns_ trace-all $tracefd

# define topology
$topo load_flatgrid $val(x) $val(y)

#
# Create God
#
set god_ [create-god $val(nn)]

# Global node setting

$ns_ node-config -adhocRouting $val(adhocRouting) \
                 -llType $val(ll) \
                 -macType $val(mac) \
                 -ifqType $val(ifq) \
                 -ifqLen $val(ifqlen) \
                 -antType $val(ant) \
                 -propType $val(prop) \
                 -phyType $val(netif) \
                 -channelType $val(chan) \
		 -topoInstance $topo \
		 -agentTrace ON \
                 -routerTrace OFF \
                 -macTrace OFF 


#  Create the specified number of nodes [$val(nn)] 

for {set i 0} {$i < $val(nn) } {incr i} {
	set node_($i) [$ns_ node]	
	$node_($i) random-motion 0		;# disable random motion
}

 
# Define node movement model

puts "Loading connection pattern..."
source $val(motionFile)

############################################################################## 

# Set up SLP servers (nos: 10-19)

for {set i 0} {$i < $val(sn) } {incr i} {
  set j [get_snode $nnode_ array]
  set udp_sa_($i)  [new Agent/UDP/UDPslp]
  $ns_ attach-agent $node_($j) $udp_sa_($i)
  $node_($j) attach $udp_sa_($i) $val(SLPport)
  set slpsa_($i) [new Application/SLPsa]
  $slpsa_($i) attach-agent $udp_sa_($i)
  $ns_ at 0.0100000000  "$node_($j) bcast-join-group $val(MulticastAddr)"
  $ns_ at 0.0 "$slpsa_($i) start"
}
#############################################################################
# Set up SLP clients (nos: 0-9)

for {set i 0} {$i < $val(cn) } {incr i} {
  set j [get_cnode $nnode_ array]
  #Setup a SLP UDP connection
  set udp_ua_($i)  [new Agent/UDP/UDPslp]
  $ns_ attach-agent $node_($j) $udp_ua_($i) 
  $node_($j) attach $udp_ua_($i) $val(SLPport)
  set slpua_($i) [new Application/SLPua]
  $slpua_($i) set maxSearch_ $val(maxSearch)  
  $slpua_($i) set dstport_ $val(SLPport)
  $slpua_($i) attach-agent $udp_ua_($i)
  $slpua_($i) add-scope DEFAULT
  $ns_ at 0.0 "$slpua_($i) start"
}

for {set i 0} {$i < $val(nn) } {incr i} {
  if {$array($i) == 2} {
    for {set j 0} {$j < $val(nn) } {incr j} {
      if {$array($j) == 2} {
        $ns_ at 0.0100000000  "$node_($i) bcast-join-group [expr $val(MulticastAddr)+$j+1]" 
      }
    }
  }
}
#############################################################################
# Tell nodes when the simulation ends

for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at [expr $val(stop)+$val(extra)] "$node_($i) reset";
}

$ns_ at [expr $val(stop)+$val(extra)+0.002] "puts \"NS EXITING...$val(scenario)\" ; $ns_ halt"

# Output simulation parameters first to trace file

puts $tracefd "# Scenario: $val(scenario)"
puts $tracefd "# Nodes: $val(nn)"
puts $tracefd "# Server Agents: $val(sn)"
puts $tracefd "# Duplicate Servers: $val(dup)"
puts $tracefd "# Client Agents: $val(cn)"
puts $tracefd "# Total Requests: $val(reqs)"
puts $tracefd "# Simultaneous Requests: $val(simult)"
puts $tracefd "# Max Search: $val(maxSearch)"
puts $tracefd "# Service lifetime: $val(lifetime) s"
puts $tracefd "# Interarrival exponential avg: $val(exp_avg) s"
puts $tracefd "# Area: $val(x)m X $val(y)m"
puts $tracefd "# Routing: $val(adhocRouting)"
puts $tracefd "# Simulation Time: $val(stop) s  (first 1000.0 s discarded)"

############################################################################## 
# Add service to server agents
for {set i 0} {$i < $val(a_sn) } {incr i} {
  $slpsa_($i) add-service service:ServiceNum$i://134.117.61.$i $val(stop) $val(lifetime)
}
for {set i $val(a_sn)} {$i < $val(sn) } {incr i} {
  $slpsa_($i) add-service service:ServiceNum$val(a_sn)://134.117.61.$val(a_sn) $val(stop) $val(lifetime)
}  
############################################################################## 

# Clients send requests
set j 0
set simultaneous [get_time $arrival_ $val(stop)]
for {set i 0} {$i < $val(reqs) } {incr i} {
   set sno [expr round([$service_ value])]
   if {$j<$val(simult)}  {
    $ns_ at $simultaneous "$slpua_($j) rqst-service service:ServiceNum$sno $val(MulticastAddr)"
  } elseif {$j==$val(simult)} {
    set simultaneous [get_time $arrival_ $val(stop)]
    $ns_ at [get_time $arrival_ $val(stop)] "$slpua_($j) rqst-service service:ServiceNum$sno $val(MulticastAddr)"
  }  else {  
    $ns_ at [get_time $arrival_ $val(stop)] "$slpua_($j) rqst-service service:ServiceNum$sno $val(MulticastAddr)"
  }
  incr j
  if {$j==$val(cn)} {
    set j 0
  }
}

############################################################################## 
# Start the Simulation
puts "Starting Simulation..."
$ns_ run


